package controller

import (
	"github.com/gin-gonic/gin"
	"icloudapp.cn/tools/entity"
	"icloudapp.cn/tools/model"
	string2 "icloudapp.cn/tools/util/string"
)

// Info 获取用户信息
func Info(ctx *gin.Context) {
	uid := string2.ConvertInt64(ctx.Query("uid"))

	if uid < 1 {
		ResponseError(ctx, entity.CodeInvalidParam)
		return
	}
	mUser := model.NewUser(ctx)
	userInfo, err := mUser.Info(uid)

	if err != nil {
		ResponseNormal(ctx, userInfo.Code, userInfo.Msg, nil)
		return
	}

	ResponseSuccess(ctx, userInfo.Body)
}

// Login 登录
func Login(ctx *gin.Context) {
	var request entity.LoginRequest
	if err := ctx.ShouldBind(&request); err != nil {
		ResponseError(ctx, entity.CodeInvalidParam)
		return
	}
	mUser := model.NewUser(ctx)
	loginInfo, err := mUser.Login(request.UserName, request.Password)

	if err != nil {
		ResponseNormal(ctx, entity.CodeServerBusy, "系统错误", nil)
	}

	if loginInfo.Uid < 1 {
		ResponseNormal(ctx, entity.CodeDenied, "登录失败", []interface{}{})
		return
	}
	ResponseSuccess(ctx, loginInfo)
}

// Register 注册
func Register(ctx *gin.Context) {
	var request entity.RegisterRequest

	if err := ctx.ShouldBind(&request); err != nil {
		ResponseError(ctx, entity.CodeInvalidParam)
		return
	}

	mUser := model.NewUser(ctx)

	userInfo, err := mUser.Register(request.UserName, request.Password, request.Email)

	if err != nil {
		ResponseNormal(ctx, userInfo.Code, err.Error(), []interface{}{})
		return
	}
	if userInfo.Code > 0 {
		//ResponseNormal(ctx, entity.CodeDenied, userInfo.Msg, userInfo.Body)
		ResponseNormal(ctx, userInfo.Code, userInfo.Msg, []interface{}{})
		return
	}
	ResponseSuccess(ctx, userInfo)
}
