package entity

import "errors"

type ResCode int64

const (
	CodeSuccess         ResCode = 0
	CodeNotAllowed      ResCode = 405
	CodeDenied          ResCode = 403
	CodePageNotFound    ResCode = 404
	CodeInvalidParam    ResCode = 1001
	CodeUserExist       ResCode = 1002
	CodeUserNotExist    ResCode = 1003
	CodeInvalidPassword ResCode = 1004
	CodeServerBusy      ResCode = 1005
	CodeNeedLogin       ResCode = 1006
	CodeInvalidToken    ResCode = 1007

	CodeEmailExist    ResCode = 1008
	CodeEmailNotExist ResCode = 1009

	CodeFileNotExist ResCode = 1010

	CodeSystemError      ResCode = 1011
	CodeDataDoesNotExist ResCode = 1012
)

var codeMsgMap = map[ResCode]string{
	CodeSuccess:          "success",
	CodeNotAllowed:       "不被允许",
	CodeDenied:           "拒绝访问",
	CodePageNotFound:     "404 Not Found",
	CodeInvalidParam:     "请求参数错误",
	CodeUserExist:        "用户已存在",
	CodeUserNotExist:     "用户不存在",
	CodeInvalidPassword:  "用户名或密码错误",
	CodeServerBusy:       "服务繁忙",
	CodeNeedLogin:        "需要登录",
	CodeInvalidToken:     "无效的token",
	CodeEmailExist:       "邮箱已注册",
	CodeEmailNotExist:    "邮箱不存在",
	CodeFileNotExist:     "文件不存在",
	CodeSystemError:      "系统错误",
	CodeDataDoesNotExist: "未找到相关数据",
}

type Code struct {
	Code ResCode
	Msg  string
	Body interface{}
}
type Response struct {
	Code
	Body interface {
	}
}

func (c ResCode) Msg() string {
	msg, ok := codeMsgMap[c]
	if !ok {
		msg = codeMsgMap[CodeServerBusy]
	}
	return msg
}

func (c ResCode) Error() error {
	return errors.New(c.Msg())
}
