package service

import (
	"context"
	"errors"
	"icloudapp.cn/tools/entity"
	"icloudapp.cn/tools/repository/mysql"
	"icloudapp.cn/tools/util"
)

type UserPoster struct {
	ctx context.Context
}

func NewPoster(ctx context.Context) *UserPoster {
	return &UserPoster{ctx}
}

func (p *UserPoster) TableName() string {
	return "user_poster"
}
func (p *UserPoster) InfoByID(posterID int64) (entity.UserPosterInfo, error) {
	var userPosterInfo entity.UserPosterInfo
	var userPoster entity.UserPoster

	userPoster.PosterID = posterID
	result := mysql.DBConn.Where("poster_id = ?", posterID).Table(p.TableName()).Find(&userPoster)
	if result.Error != nil {
		userPosterInfo.Code = entity.CodeServerBusy
		userPosterInfo.Msg = entity.CodeServerBusy.Msg()
		return userPosterInfo, errors.New(result.Error.Error())
	}
	if result.RowsAffected == 0 {
		userPosterInfo.Code = entity.CodeDataDoesNotExist
		userPosterInfo.Msg = entity.CodeDataDoesNotExist.Msg()
		return userPosterInfo, util.NewError("未找到相关数据")
	}
	userPosterInfo.Body = userPoster
	return userPosterInfo, nil
}

func (p *UserPoster) InfoByUUID(uuid int64) (entity.UserPosterInfo, error) {
	var userPosterInfo entity.UserPosterInfo
	var userPoster entity.UserPoster

	userPoster.PosterID = uuid
	result := mysql.DBConn.Where("uuid = ?", uuid).Table(p.TableName()).Find(&userPoster)
	if result.Error != nil {
		userPosterInfo.Code = entity.CodeServerBusy
		userPosterInfo.Msg = entity.CodeServerBusy.Msg()
		return userPosterInfo, errors.New(result.Error.Error())
	}
	if result.RowsAffected == 0 {
		userPosterInfo.Code = entity.CodeDataDoesNotExist
		userPosterInfo.Msg = entity.CodeDataDoesNotExist.Msg()
		return userPosterInfo, util.NewError("未找到相关数据")
	}
	userPosterInfo.Body = userPoster
	return userPosterInfo, nil
}
