package image

import (
	"fmt"
	"github.com/skip2/go-qrcode"
	"gopkg.in/gographics/imagick.v3/imagick"
	"icloudapp.cn/tools/util"
	"image/color"
	"sync"
)

type QR struct {
	Cache
	Value           string
	FrontColor      string
	BackgroundColor string
	QRSize          *Size
}

var QROnce sync.Once
var QRInstance *QR

func NewQR() *QR {
	QROnce.Do(func() {
		QRInstance = &QR{}
	})
	return QRInstance
}

func (q *QR) SetCache(path string) {
	q.Cache.SetCache(path)
}

func (q *QR) SetValue(value string) {
	q.Value = value
}

func (q *QR) SetFrontColor(frontColor string) {
	q.FrontColor = frontColor
}

func (q *QR) SetBackgroundColor(backgroundColor string) {
	q.BackgroundColor = backgroundColor
}
func (q *QR) SetSize(width, height uint) {
	q.QRSize = NewSize(width, height)
}
func (q *QR) Create() (*imagick.MagickWand, error) {
	//file := fmt.Sprintf("%s.png", util.Sign(q.Value, "QR"))
	file := q.CacheFile(util.Sign(q.Value, "QR"), "png")
	if q.BackgroundColor == "" {
		q.BackgroundColor = "#000"
	}
	if q.FrontColor == "" {
		q.FrontColor = "#fff"
	}
	_, background, _ := Alpha(q.BackgroundColor)
	_, front, _ := Alpha(q.FrontColor)

	background.A = 1
	backgroundColor := color.RGBA{R: background.R, G: background.G, B: background.B, A: background.A}

	front.A = 1
	frontColor := color.RGBA{R: front.R, G: front.G, B: front.B, A: front.A}

	if q.QRSize.Width == 0 {
		q.QRSize.Width = 200
	}

	qr, err := qrcode.New(q.Value, qrcode.Medium)
	if err != nil {
		return nil, err
	}

	qr.Image(int(q.QRSize.Width))
	qr.DisableBorder = true

	qr.ForegroundColor = color.RGBA{R: 0x33, G: 0x33, B: 0x66, A: 0xff}
	qr.BackgroundColor = color.RGBA{R: 0xef, G: 0xef, B: 0xef, A: 0xff}

	//qr.ForegroundColor = frontColor
	//qr.BackgroundColor = backgroundColor
	fmt.Println("background", backgroundColor, frontColor)

	if err = qr.WriteFile(int(q.QRSize.Width), file); err != nil {
		return nil, util.NewError("qrcode.WriteColorFile", err.Error())
	}
	/*if err := qrcode.WriteColorFile(q.Value, qrcode.Medium, int(q.QRSize.Width), color.White, color.Black, file); err != nil {
		return nil, util.NewError("qrcode.WriteColorFile", err.Error())
	}*/

	qrWM := imagick.NewMagickWand()
	if err = qrWM.ReadImage(file); err != nil {
		return nil, util.NewError("qrWM.SetImageFilename", err.Error())
	}
	if err = qrWM.SetSize(q.QRSize.Width, q.QRSize.Height); err != nil {
		return nil, util.NewError("qrWM.SetSize", err.Error())
	}

	return qrWM, nil
}
